@@ -75,7 +75,7 @@ func DefineBuildFlags(cmd *cobra.Command, buildOpts *BuildFlagsWrapper, isFarmBu
 	if err := flag.Value.Set("missing"); err != nil {
 		logrus.Errorf("Unable to set --pull to 'missing': %v", err)
 	}
-	flag.Usage = `Pull image policy ("always/true"|"missing"|"never/false"|"newer")`
+	flag.Usage = `Pull image policy ("always"|"missing"|"never"|"newer")`
 	flags.AddFlagSet(&budFlags)
 
 	// Add the completion functions
@@ -136,9 +136,8 @@ func DefineBuildFlags(cmd *cobra.Command, buildOpts *BuildFlagsWrapper, isFarmBu
 }
 
 func ParseBuildOpts(cmd *cobra.Command, args []string, buildOpts *BuildFlagsWrapper) (*entities.BuildOptions, error) {
-	if (cmd.Flags().Changed("squash") && cmd.Flags().Changed("layers")) ||
-		(cmd.Flags().Changed("squash-all") && cmd.Flags().Changed("squash")) {
-		return nil, errors.New("cannot specify --squash with --layers and --squash-all with --squash")
+	if cmd.Flags().Changed("squash-all") && cmd.Flags().Changed("squash") {
+		return nil, errors.New("cannot specify --squash-all with --squash")
 	}
 
 	if cmd.Flag("output").Changed && registry.IsRemote() {
@@ -203,10 +202,7 @@ func ParseBuildOpts(cmd *cobra.Command, args []string, buildOpts *BuildFlagsWrap
 		// No context directory or URL was specified.  Try to use the home of
 		// the first locally-available Containerfile.
 		for i := range containerFiles {
-			if strings.HasPrefix(containerFiles[i], "http://") ||
-				strings.HasPrefix(containerFiles[i], "https://") ||
-				strings.HasPrefix(containerFiles[i], "git://") ||
-				strings.HasPrefix(containerFiles[i], "github.com/") {
+			if isURL(containerFiles[i]) {
 				continue
 			}
 			absFile, err := filepath.Abs(containerFiles[i])
@@ -242,6 +238,10 @@ func ParseBuildOpts(cmd *cobra.Command, args []string, buildOpts *BuildFlagsWrap
 		}
 	}
 
+	if err := areContainerfilesValid(contextDir, containerFiles); err != nil {
+		return nil, err
+	}
+
 	var logFile *os.File
 	if cmd.Flag("logfile").Changed {
 		var err error
@@ -520,6 +520,7 @@ func buildFlagsWrapperToOptions(c *cobra.Command, contextDir string, flags *Buil
 		CacheTTL:                cacheTTL,
 		ConfidentialWorkload:    confidentialWorkloadOptions,
 		CommonBuildOpts:         commonOpts,
+		CompatVolumes:           types.NewOptionalBool(flags.CompatVolumes),
 		Compression:             compression,
 		ConfigureNetwork:        networkPolicy,
 		ContextDirectory:        contextDir,
@@ -629,3 +630,41 @@ func parseDockerignore(ignoreFile string) ([]string, error) {
 	}
 	return excludes, nil
 }
+
+func areContainerfilesValid(contextDir string, containerFiles []string) error {
+	for _, f := range containerFiles {
+		if isURL(f) || f == "/dev/stdin" {
+			continue
+		}
+
+		// Because currently podman runs the test/bud.bats tests under the buildah project in CI,
+		// the following error messages need to be consistent with buildah; otherwise, the podman CI will fail.
+		// See: https://github.com/containers/buildah/blob/4c781b59b49d66e07324566555339888113eb7e2/imagebuildah/build.go#L139-L141
+		// 	    https://github.com/containers/buildah/blob/4c781b59b49d66e07324566555339888113eb7e2/tests/bud.bats#L3474-L3479
+		if utils.IsDir(f) {
+			return fmt.Errorf("containerfile: %q cannot be path to a directory", f)
+		}
+
+		// If the file is not found, try again with context directory prepended (if not prepended yet)
+		// Ref: https://github.com/containers/buildah/blob/4c781b59b49d66e07324566555339888113eb7e2/imagebuildah/build.go#L125-L135
+		if utils.FileExists(f) {
+			continue
+		}
+		if !strings.HasPrefix(f, contextDir) {
+			if utils.FileExists(filepath.Join(contextDir, f)) {
+				continue
+			}
+		}
+
+		return fmt.Errorf("the specified Containerfile or Dockerfile does not exist, %s: %w", f, syscall.ENOENT)
+	}
+
+	return nil
+}
+
+func isURL(s string) bool {
+	return strings.HasPrefix(s, "http://") ||
+		strings.HasPrefix(s, "https://") ||
+		strings.HasPrefix(s, "git://") ||
+		strings.HasPrefix(s, "github.com/")
+}
