@@ -49,12 +49,13 @@ var (
 	// Key: Extension
 	// Value: Processing order for resource naming dependencies
 	supportedExtensions = map[string]int{
-		".container": 3,
+		".container": 4,
 		".volume":    2,
-		".kube":      3,
+		".kube":      4,
 		".network":   2,
 		".image":     1,
-		".pod":       4,
+		".build":     3,
+		".pod":       5,
 	}
 )
 
@@ -157,9 +158,12 @@ func appendSubPaths(dirs []string, path string, isUserFlag bool, filterPtr func(
 	}
 
 	err = filepath.WalkDir(resolvedPath, func(_path string, info os.DirEntry, err error) error {
-		if info == nil || info.IsDir() {
-			if filterPtr == nil || filterPtr(_path, isUserFlag) {
-				dirs = append(dirs, _path)
+		// Ignore drop-in directory subpaths
+		if !strings.HasSuffix(_path, ".d") {
+			if info == nil || info.IsDir() {
+				if filterPtr == nil || filterPtr(_path, isUserFlag) {
+					dirs = append(dirs, _path)
+				}
 			}
 		}
 		return err
@@ -255,16 +259,11 @@ func loadUnitDropins(unit *parser.UnitFile, sourcePaths []string) error {
 	}
 
 	dropinDirs := []string{}
+	unitDropinPaths := unit.GetUnitDropinPaths()
 
 	for _, sourcePath := range sourcePaths {
-		dropinDirs = append(dropinDirs, path.Join(sourcePath, unit.Filename+".d"))
-	}
-
-	// For instantiated templates, also look in the non-instanced template dropin dirs
-	templateBase, templateInstance := unit.GetTemplateParts()
-	if templateBase != "" && templateInstance != "" {
-		for _, sourcePath := range sourcePaths {
-			dropinDirs = append(dropinDirs, path.Join(sourcePath, templateBase+".d"))
+		for _, dropinPath := range unitDropinPaths {
+			dropinDirs = append(dropinDirs, path.Join(sourcePath, dropinPath))
 		}
 	}
 
@@ -358,15 +357,14 @@ func enableServiceFile(outputPath string, service *parser.UnitFile) {
 	}
 
 	serviceFilename := service.Filename
-	templateBase, templateInstance := service.GetTemplateParts()
+	templateBase, templateInstance, isTemplate := service.GetTemplateParts()
 
 	// For non-instantiated template service we only support installs if a
 	// DefaultInstance is given. Otherwise we ignore the Install group, but
 	// it is still useful when instantiating the unit via a symlink.
-	if templateBase != "" && templateInstance == "" {
+	if isTemplate && templateInstance == "" {
 		if defaultInstance, ok := service.Lookup(quadlet.InstallGroup, "DefaultInstance"); ok {
-			parts := strings.SplitN(templateBase, "@", 2)
-			serviceFilename = parts[0] + "@" + defaultInstance + parts[1]
+			serviceFilename = templateBase + "@" + defaultInstance + filepath.Ext(serviceFilename)
 		} else {
 			serviceFilename = ""
 		}
@@ -474,7 +472,7 @@ func warnIfAmbiguousName(unit *parser.UnitFile, group string) {
 	if !ok {
 		return
 	}
-	if strings.HasSuffix(imageName, ".image") {
+	if strings.HasSuffix(imageName, ".build") || strings.HasSuffix(imageName, ".image") {
 		return
 	}
 	if !isUnambiguousName(imageName) {
@@ -499,6 +497,19 @@ func generatePodsInfoMap(units []*parser.UnitFile) map[string]*quadlet.PodInfo {
 	return podsInfoMap
 }
 
+func prefillBuiltImageNames(units []*parser.UnitFile, resourceNames map[string]string) {
+	for _, unit := range units {
+		if !strings.HasSuffix(unit.Filename, ".build") {
+			continue
+		}
+
+		imageName := quadlet.GetBuiltImageName(unit)
+		if len(imageName) > 0 {
+			resourceNames[unit.Filename] = imageName
+		}
+	}
+}
+
 func main() {
 	if err := process(); err != nil {
 		Logf("%s", err.Error())
@@ -600,6 +611,12 @@ func process() error {
 	// A map of network/volume unit file-names, against their calculated names, as needed by Podman.
 	var resourceNames = make(map[string]string)
 
+	// Prefill resouceNames for .build files. This is significantly less complex than
+	// pre-computing all resourceNames for all Quadlet types (which is rather complex for a few
+	// types), but still breaks the dependency cycle between .volume and .build ([Volume] can
+	// have Image=some.build, and [Build] can have Volume=some.volume:/some-volume)
+	prefillBuiltImageNames(units, resourceNames)
+
 	for _, unit := range units {
 		var service *parser.UnitFile
 		var name string
@@ -619,6 +636,8 @@ func process() error {
 		case strings.HasSuffix(unit.Filename, ".image"):
 			warnIfAmbiguousName(unit, quadlet.ImageGroup)
 			service, name, err = quadlet.ConvertImage(unit)
+		case strings.HasSuffix(unit.Filename, ".build"):
+			service, name, err = quadlet.ConvertBuild(unit, resourceNames)
 		case strings.HasSuffix(unit.Filename, ".pod"):
 			service, err = quadlet.ConvertPod(unit, unit.Filename, podsInfoMap, resourceNames)
 		default:
