@@ -281,6 +281,12 @@ func Build(ctx context.Context, containerFiles []string, options types.BuildOpti
 	if mem := options.CommonBuildOpts.Memory; mem > 0 {
 		params.Set("memory", strconv.Itoa(int(mem)))
 	}
+	switch options.CompatVolumes {
+	case imageTypes.OptionalBoolTrue:
+		params.Set("compatvolumes", "1")
+	case imageTypes.OptionalBoolFalse:
+		params.Set("compatvolumes", "0")
+	}
 	if options.NoCache {
 		params.Set("nocache", "1")
 	}
@@ -483,7 +489,7 @@ func Build(ctx context.Context, containerFiles []string, options types.BuildOpti
 			dontexcludes = append(dontexcludes, "!"+containerfile+".containerignore")
 		} else {
 			// If Containerfile does not exist, assume it is in context directory and do Not add to tarfile
-			if _, err := os.Lstat(containerfile); err != nil {
+			if err := fileutils.Lexists(containerfile); err != nil {
 				if !os.IsNotExist(err) {
 					return nil, err
 				}
