@@ -5,6 +5,10 @@ import "os"
 const UserCertsTargetPath = "/etc/containers/certs.d"
 const DefaultIdentityName = "machine"
 
+// MountTag is an identifier to mount a VirtioFS file system tag on a mount point in the VM.
+// Ref: https://developer.apple.com/documentation/virtualization/running_intel_binaries_in_linux_vms_with_rosetta
+const MountTag = "rosetta"
+
 var (
 	DefaultFilePerm os.FileMode = 0644
 )
